@@ -111,6 +111,7 @@
 #define API_SERVER_METHOD		"POST"
 #define API_SERVER_URI_ADD		"/api/scores/add"
 #define API_SERVER_URI_GET		"/api/scores/get"
+#define API_SERVER_URI_GETTAPE		"/api/scores/gettape"
 #define API_SERVER_URI_RENAME		"/api/players/rename"
 #define API_SERVER_URI_RESETUUID	"/api/players/resetuuid"
 
@@ -175,6 +176,8 @@
 // default shortcut keys
 #define DEFAULT_KEY_SAVE_GAME		KSYM_F1
 #define DEFAULT_KEY_LOAD_GAME		KSYM_F2
+#define DEFAULT_KEY_RESTART_GAME	KSYM_F3
+#define DEFAULT_KEY_PAUSE_BEFORE_END	KSYM_F4
 #define DEFAULT_KEY_TOGGLE_PAUSE	KSYM_space
 #define DEFAULT_KEY_FOCUS_PLAYER_1	KSYM_F5
 #define DEFAULT_KEY_FOCUS_PLAYER_2	KSYM_F6
@@ -652,10 +655,14 @@
 #define TAPES_DIRECTORY		"tapes"
 #define SCORES_DIRECTORY	"scores"
 #define DOCS_DIRECTORY		"docs"
+#define ELEMENTS_DIRECTORY	"elements"
+#define CREDITS_DIRECTORY	"credits"
+#define INFO_DIRECTORY		"info"
 #define CACHE_DIRECTORY		"cache"
 #define CONF_DIRECTORY		"conf"
 #define NETWORK_DIRECTORY	"network"
 #define USERS_DIRECTORY		"users"
+#define PERSISTENT_DIRECTORY	"/persistent"
 
 #define GFX_CLASSIC_SUBDIR	"gfx_classic"
 #define SND_CLASSIC_SUBDIR	"snd_classic"
@@ -789,9 +796,10 @@
 #define TREE_TYPE_LEVEL_DIR	3
 #define TREE_TYPE_LEVEL_NR	4
 #define TREE_TYPE_PLAYER_NAME	5
+#define TREE_TYPE_SCORE_ENTRY	6
 
 #define NUM_BASE_TREE_TYPES	4
-#define NUM_TREE_TYPES		6
+#define NUM_TREE_TYPES		7
 
 #define TREE_TYPE_IS_DIR(type)	((type) == TREE_TYPE_GRAPHICS_DIR ||	\
 				 (type) == TREE_TYPE_SOUNDS_DIR ||	\
@@ -805,12 +813,16 @@
 #define INFOTEXT_LEVEL_DIR	"Level Sets"
 #define INFOTEXT_LEVEL_NR	"Levels"
 #define INFOTEXT_PLAYER_NAME	"Players & Teams"
+#define INFOTEXT_SCORE_ENTRY	"Hall of Fame"
 
 #define BACKLINK_TEXT_MAIN	".. (main menu)"
 #define BACKLINK_TEXT_SETUP	".. (setup menu)"
 #define BACKLINK_TEXT_PARENT	".. (parent directory)"
+#define BACKLINK_TEXT_SCORES	"back"
 
-#define TREE_INFOTEXT(t)	((t) == TREE_TYPE_PLAYER_NAME ?		\
+#define TREE_INFOTEXT(t)	((t) == TREE_TYPE_SCORE_ENTRY ?		\
+				 INFOTEXT_SCORE_ENTRY :			\
+				 (t) == TREE_TYPE_PLAYER_NAME ?		\
 				 INFOTEXT_PLAYER_NAME :			\
 				 (t) == TREE_TYPE_LEVEL_NR ?		\
 				 INFOTEXT_LEVEL_NR :			\
@@ -824,7 +836,9 @@
 				 INFOTEXT_MUSIC_DIR :			\
 				 INFOTEXT_UNDEFINED)
 
-#define TREE_BACKLINK_TEXT(t)	((t) == TREE_TYPE_LEVEL_DIR ?		\
+#define TREE_BACKLINK_TEXT(t)	((t) == TREE_TYPE_SCORE_ENTRY ?		\
+				 BACKLINK_TEXT_SCORES :			\
+				 (t) == TREE_TYPE_LEVEL_DIR ?		\
 				 BACKLINK_TEXT_MAIN :			\
 				 BACKLINK_TEXT_SETUP)
 
@@ -985,7 +999,12 @@
 #define UPDATE_BUSY_STATE()			\
 {						\
   if (gfx.draw_busy_anim_function != NULL)	\
-    gfx.draw_busy_anim_function();		\
+    gfx.draw_busy_anim_function(TRUE);		\
+}
+#define UPDATE_BUSY_STATE_NOT_LOADING()		\
+{						\
+  if (gfx.draw_busy_anim_function != NULL)	\
+    gfx.draw_busy_anim_function(FALSE);		\
 }
 
 
@@ -1207,7 +1226,7 @@ struct GfxInfo
 
   int anim_random_frame;
 
-  void (*draw_busy_anim_function)(void);
+  void (*draw_busy_anim_function)(boolean);
   void (*draw_global_anim_function)(int, int);
   void (*draw_global_border_function)(int);
   void (*draw_tile_cursor_function)(int);
@@ -1365,6 +1384,8 @@ struct SetupShortcutInfo
 {
   Key save_game;
   Key load_game;
+  Key restart_game;
+  Key pause_before_end;
   Key toggle_pause;
 
   Key focus_player[MAX_PLAYERS];
@@ -1435,6 +1456,15 @@ struct SetupInternalInfo
   boolean menu_shortcuts;
   boolean menu_exit;
   boolean menu_save_and_exit;
+
+  boolean info_title;
+  boolean info_elements;
+  boolean info_music;
+  boolean info_credits;
+  boolean info_program;
+  boolean info_version;
+  boolean info_levelset;
+  boolean info_exit;
 };
 
 struct SetupDebugInfo
@@ -1468,6 +1498,7 @@ struct SetupInfo
   int engine_snapshot_memory;
   boolean fade_screens;
   boolean autorecord;
+  boolean auto_pause_on_start;
   boolean show_titlescreen;
   boolean quick_doors;
   boolean team_mode;
@@ -1592,10 +1623,14 @@ struct TreeInfo
 
   char *special_flags;	// flags for special actions performed on level file
 
+  char *empty_level_name;   // name pattern if level title is "nameless level"
+  boolean force_level_name; // force also renaming non-nameless level titles
+
   int levels;		// number of levels in level series
   int first_level;	// first level number (to allow start with 0 or 1)
   int last_level;	// last level number (automatically calculated)
   int sort_priority;	// sort levels by 'sort_priority' and then by name
+  int pos;		// custom position information of node in tree
 
   boolean latest_engine;// force level set to use the latest game engine
 
@@ -1914,7 +1949,7 @@ void InitGfxDoor3Info(int, int, int, int);
 void InitGfxWindowInfo(int, int);
 void InitGfxScrollbufferInfo(int, int);
 void InitGfxClipRegion(boolean, int, int, int, int);
-void InitGfxDrawBusyAnimFunction(void (*draw_busy_anim_function)(void));
+void InitGfxDrawBusyAnimFunction(void (*draw_busy_anim_function)(boolean));
 void InitGfxDrawGlobalAnimFunction(void (*draw_global_anim_function)(int, int));
 void InitGfxDrawGlobalBorderFunction(void (*draw_global_border_function)(int));
 void InitGfxDrawTileCursorFunction(void (*draw_tile_cursor_function)(int));
@@ -2014,6 +2049,7 @@ KeyMod GetKeyModStateFromEvents(void);
 void StartTextInput(int, int, int, int);
 void StopTextInput(void);
 void PushUserEvent(int, int, int);
+boolean PendingEscapeKeyEvent(void);
 
 void InitJoysticks(void);
 boolean ReadJoystick(int, int *, int *, boolean *, boolean *);
