@@ -263,15 +263,20 @@ void TapeStartPlaying(void);
 void TapeStopPlaying(void);
 byte *TapePlayAction(void);
 void TapeStop(void);
+void TapeStopGame(void);
+void TapeStopTape(void);
 void TapeErase(void);
 unsigned int GetTapeLengthFrames(void);
 unsigned int GetTapeLengthSeconds(void);
 void TapeQuickSave(void);
 void TapeQuickLoad(void);
+void TapeRestartGame(void);
+void TapeReplayAndPauseBeforeEnd(void);
 
 boolean hasSolutionTape(void);
 boolean InsertSolutionTape(void);
 boolean PlaySolutionTape(void);
+boolean PlayScoreTape(int);
 
 void UndoTape(void);
 void FixTape_ForceSinglePlayer(void);
