@@ -131,6 +131,7 @@ void FloodFillLevelExt(int, int, int, int, int y, short field[][y], int, int);
 
 void SetRandomAnimationValue(int, int);
 int getGraphicAnimationFrame(int, int);
+int getGraphicAnimationFrameXY(int, int, int);
 
 void DrawFixedGraphicAnimation(int, int, int);
 void DrawFixedGraphicAnimationExt(DrawBuffer *, int, int, int, int, int);
@@ -150,6 +151,7 @@ void getSizedGraphicSourceExt(int, int, int, Bitmap **, int *, int *, boolean);
 void getSizedGraphicSource(int, int, int, Bitmap **, int *, int *);
 void getFixedGraphicSource(int, int, Bitmap **, int *, int *);
 void getMiniGraphicSource(int, Bitmap **, int *, int *);
+void getGlobalAnimGraphicSource(int, int, Bitmap **, int *, int *);
 void getGraphicSource(int, int, Bitmap **, int *, int *);
 
 void DrawGraphic(int, int, int, int);
@@ -180,6 +182,8 @@ void DrawLevelFieldThruMask(int, int);
 void DrawLevelFieldCrumbled(int, int);
 void DrawLevelFieldCrumbledDigging(int, int, int, int);
 void DrawLevelFieldCrumbledNeighbours(int, int);
+void DrawScreenGraphic(int, int, int, int);
+void DrawLevelGraphic(int, int, int, int);
 void DrawScreenElement(int, int, int);
 void DrawLevelElement(int, int, int);
 void DrawScreenField(int, int);
@@ -296,4 +300,9 @@ void ChangeViewportPropertiesIfNeeded(void);
 boolean CheckIfAllViewportsHaveChanged(void);
 boolean CheckFadeAll(void);
 
+void OpenURL(char *);
+void OpenURLFromHash(SetupFileHash *, int);
+
+void TestGeneratingUUIDs(void);
+
 #endif	// TOOLS_H
